@@ -363,6 +363,53 @@ class TestSelect(TestBase):
                 '2-log': [[1471254712, 'log line two']]
             })
 
+        self.assertEqual(
+            await self.client0.query('select timeval() from "aggr"'),
+            {'aggr': [
+                [1447249033, 1447249033],
+                [1447249337, 1447249337],
+                [1447249633, 1447249633],
+                [1447249937, 1447249937],
+                [1447250249, 1447250249],
+                [1447250549, 1447250549],
+                [1447250868, 1447250868],
+                [1447251168, 1447251168],
+                [1447251449, 1447251449],
+                [1447251749, 1447251749],
+                [1447252049, 1447252049],
+                [1447252349, 1447252349],
+                [1447252649, 1447252649],
+                [1447252968, 1447252968],
+                [1447253244, 1447253244],
+                [1447253549, 1447253549],
+                [1447253849, 1447253849],
+                [1447254149, 1447254149],
+                [1447254449, 1447254449],
+                [1447254748, 1447254748]]})
+
+        self.assertEqual(
+            await self.client0.query('select interval() from "aggr"'),
+            {'aggr': [
+                [1447249337, 304],
+                [1447249633, 296],
+                [1447249937, 304],
+                [1447250249, 312],
+                [1447250549, 300],
+                [1447250868, 319],
+                [1447251168, 300],
+                [1447251449, 281],
+                [1447251749, 300],
+                [1447252049, 300],
+                [1447252349, 300],
+                [1447252649, 300],
+                [1447252968, 319],
+                [1447253244, 276],
+                [1447253549, 305],
+                [1447253849, 300],
+                [1447254149, 300],
+                [1447254449, 300],
+                [1447254748, 299]]})
+
         self.assertEqual(
             await self.client0.query('select difference() from "one"'),
             {'one': []})
