@@ -26,6 +26,7 @@ static siri_cfg_t siri_cfg = {
         .optimize_interval=3600,
         .ip_support=IP_SUPPORT_ALL,
         .shard_compression=0,
+        .shard_auto_duration=0,
         .server_address="localhost",
         .default_db_path="",
         .pipe_support=0,
@@ -53,6 +54,7 @@ static void SIRI_CFG_read_default_db_path(cfgparser_t * cfgparser);
 static void SIRI_CFG_read_max_open_files(cfgparser_t * cfgparser);
 static void SIRI_CFG_read_ip_support(cfgparser_t * cfgparser);
 static void SIRI_CFG_read_shard_compression(cfgparser_t * cfgparser);
+static void SIRI_CFG_read_shard_auto_duration(cfgparser_t * cfgparser);
 static void SIRI_CFG_read_pipe_support(cfgparser_t * cfgparser);
 
 void siri_cfg_init(siri_t * siri)
@@ -130,6 +132,7 @@ void siri_cfg_init(siri_t * siri)
     SIRI_CFG_read_max_open_files(cfgparser);
     SIRI_CFG_read_ip_support(cfgparser);
     SIRI_CFG_read_shard_compression(cfgparser);
+    SIRI_CFG_read_shard_auto_duration(cfgparser);
 
     SIRI_CFG_read_addr(
             cfgparser,
@@ -293,6 +296,35 @@ static void SIRI_CFG_read_shard_compression(cfgparser_t * cfgparser)
     }
 }
 
+static void SIRI_CFG_read_shard_auto_duration(cfgparser_t * cfgparser)
+{
+    cfgparser_option_t * option;
+    cfgparser_return_t rc;
+    rc = cfgparser_get_option(
+                &option,
+                cfgparser,
+                "siridb",
+                "enable_shard_auto_duration");
+    if (rc != CFGPARSER_SUCCESS)
+    {
+        log_warning(
+                "Missing 'enable_shard_auto_duration' in '%s' (%s).",
+                siri.args->config,
+                cfgparser_errmsg(rc));
+    }
+    else if (option->tp != CFGPARSER_TP_INTEGER || option->val->integer > 1)
+    {
+        log_warning(
+                "Error reading 'enable_shard_auto_duration' in '%s': %s.",
+                siri.args->config,
+                "error: expecting 0 or 1");
+    }
+    else if (option->val->integer == 1)
+    {
+        siri_cfg.shard_auto_duration = 1;
+    }
+}
+
 static void SIRI_CFG_read_pipe_support(cfgparser_t * cfgparser)
 {
     cfgparser_option_t * option;
