@@ -83,7 +83,7 @@ const uint8_t SERIES_SFC =
  */
 int siridb_series_cexpr_cb(siridb_series_t * series, cexpr_condition_t * cond)
 {
-    switch (cond->prop)
+    switch ((enum cleri_grammar_ids) cond->prop)
     {
     case CLERI_GID_K_LENGTH:
         return cexpr_int_cmp(cond->operator, series->length, cond->int64);
@@ -93,14 +93,20 @@ int siridb_series_cexpr_cb(siridb_series_t * series, cexpr_condition_t * cond)
         return cexpr_int_cmp(cond->operator, series->end, cond->int64);
     case CLERI_GID_K_POOL:
         return cexpr_int_cmp(cond->operator, series->pool, cond->int64);
+    case CLERI_GID_K_SHARD_DURATION:
+        return cexpr_int_cmp(
+                cond->operator,
+                (series->idx ? series->idx->shard->duration : 0),
+                cond->int64);
     case CLERI_GID_K_TYPE:
         return cexpr_int_cmp(cond->operator, series->tp, cond->int64);
     case CLERI_GID_K_NAME:
         return cexpr_str_cmp(cond->operator, series->name, cond->str);
+    default:
+        /* we must NEVER get here */
+        log_critical("Unexpected series property received: %d", cond->prop);
+        assert (0);
     }
-    /* we must NEVER get here */
-    log_critical("Unexpected series property received: %d", cond->prop);
-    assert (0);
     return -1;
 }
 
@@ -569,6 +575,7 @@ int siridb_series_flush_dropped(siridb_t * siridb)
     }
 
     siridb->groups->flags |= GROUPS_FLAG_DROPPED_SERIES;
+    siridb->tags->flags |= TAGS_FLAG_DROPPED_SERIES;
 
     return rc;
 }
@@ -585,8 +592,8 @@ void siridb_series_remove_shard(
 {
     idx_t *__restrict idx;
     uint_fast32_t i, offset;
-    uint64_t duration = (shard->tp == SIRIDB_SHARD_TP_NUMBER) ?
-                siridb->duration_num : siridb->duration_log;
+    uint64_t start = shard->id - series->mask;
+    uint64_t end = start + shard->duration;
 
     i = offset = 0;
 
@@ -632,8 +639,6 @@ void siridb_series_remove_shard(
             {
                 series->idx = idx;
             }
-            uint64_t start = shard->id - series->mask;
-            uint64_t end = start + duration;
             if (series->start >= start && series->start < end)
             {
                 SERIES_update_start(series);
@@ -1038,9 +1043,7 @@ int siridb_series_optimize_shard(
     siridb_points_t *__restrict points;
     int rc;
     uint16_t cinfo = 0;
-    uint64_t duration = (shard->tp == SIRIDB_SHARD_TP_NUMBER) ?
-                siridb->duration_num : siridb->duration_log;
-    max_ts = (shard->id + duration) - series->mask;
+    max_ts = (shard->id + shard->duration) - series->mask;
 
     rc = new_idx = end = i = size = start = 0;
 
